@@ -3,7 +3,10 @@ use strict;
 use warnings;
 use Carp;
 
-our $VERSION = '0.01_05';
+our $VERSION = '0.03_01';
+
+our $ac_opt;
+our $ac_define_class;
 
 sub import{
     my $caller_class = caller;
@@ -26,6 +29,7 @@ sub new{
     my ( $class, @args ) = @_;
     
     # check args
+    @args = %{ $args[0] } if ref $args[0] eq 'HASH';
     confess 'please pass key value pairs to new method' if @args % 2;
     
     # bless
@@ -53,7 +57,7 @@ sub ac(@){
     
     # check and rearrange accessor option;
     my $ac_opt = {};
-    my %valid_opt = map{ $_ => 1 } qw( default set_hook get_hook hash_force );
+    my %valid_opt = map{ $_ => 1 } qw( default constrain filter trigger set_hook get_hook hash_force );
         
     $ac_opt->{ default } = shift if @_ % 2;
     
@@ -63,7 +67,7 @@ sub ac(@){
     }
     
     # register accessor option
-    _SIMO_ac_opt( $ac_define_class, $key, $ac_opt );
+    $Simo::ac_opt{ $ac_define_class }{ $key } = $ac_opt;
 
     # redefine real acessor
     my $ac_redefine = qq/sub ${ac_define_class}::${key} { _SIMO_ac_real( '$key' , \@_ ) }/;
@@ -89,7 +93,10 @@ sub _SIMO_ac_real{
     my $ac_define_class = _SIMO_ac_define_class( $class, $key );
     
     # get accessor option
-    my $ac_opt = _SIMO_ac_opt( $ac_define_class, $key );
+    my $ac_opt = $Simo::ac_opt{ $ac_define_class }{ $key };
+    
+    # init by default value
+    $self->{ $key } = $ac_opt->{ default } unless exists $self->{ $key };
     
     # rearrange value;
     my $val = @vals == 1 ? $vals[0] :
@@ -98,40 +105,68 @@ sub _SIMO_ac_real{
               undef;
     
     # return value( return old_value in case setter is called )
-    my $ret = _SIMO_obj_updated($self,$key) ? $self->{$key} : $ac_opt->{default};
+    my $ret = $self->{ $key };
     
     # set value if value is defined
     if( defined( $val ) ){
         # setter hook function
+        # ( set_hook option is now not recommended. this option will be deleted in future 2019 )
         if( $ac_opt->{ set_hook } ){
             eval{ $val = $ac_opt->{ set_hook }->($self,$val) };
             confess $@ if $@;
         }
         
+        # constrain
+        my $constrains = $ac_opt->{ constrain };
+        if( $constrains ){
+            $constrains = [ $constrains ] unless ref $constrains eq 'ARRAY';
+            foreach my $constrain ( @{ $constrains } ){
+                local $_ = $val;
+                eval{ $constrain->( $val ) };
+                confess $@ if $@;
+            }
+        }
+        
+        # filter
+        my $filters = $ac_opt->{ filter };
+        if( $filters ){
+            $filters = [ $filters ] unless ref $filters eq 'ARRAY';
+            foreach my $filter ( @{ $filters } ){
+                local $_ = $val;
+                eval{ $val = $filter->( $val ) };
+                confess $@ if $@;
+            }
+        }
+        
         # set new value
         $self->{ $key } = $val;
         
-        # updated is true
-        _SIMO_obj_updated( $self, $key, 1 );
+        # trigger
+        my $triggers = $ac_opt->{ trigger };
+        if( $triggers ){
+            $triggers = [ $triggers ] unless ref $triggers eq 'ARRAY';
+            foreach my $trigger ( @{ $triggers } ){
+                local $_ = $self;
+                eval{ $trigger->( $self ) };
+                confess $@ if $@;
+            }
+        }
     }
     else{
         # getter hook function
+        # ( get_hook option is now not recommended. this option will be deleted in future 2019 )
         if( $ac_opt->{ get_hook } ){
             eval{ $ret = $ac_opt->{ get_hook }->($self, $ret) };
             confess $@ if $@;
         }
     }
-    
-    return !wantarray ? $ret :
-           ref $ret eq 'ARRAY' ? @{ $ret } :
-           ref $ret eq 'HASH' ? %{ $ret } :
-           ( $ret );
+    return $ret;
 }
 
 # Get accessor define class
 sub _SIMO_ac_define_class{
     my ( $class, $key ) = @_;
-    unless( $Simo::info{ class }{ $class }{ ac }{ $key }{ ac_define_class } ){
+    unless( $Simo::ac_define_class{ $class }{ $key } ){
         
         my $ac_define_class = ( caller 2 )[ 3 ];
         
@@ -139,9 +174,9 @@ sub _SIMO_ac_define_class{
             $ac_define_class = $1;
         }
         
-        $Simo::info{ class }{ $class }{ ac }{ $key }{ ac_define_class } = $ac_define_class;
+        $Simo::ac_define_class{ $class }{ $key } = $ac_define_class;
     }
-    return $Simo::info{ class }{ $class }{ ac }{ $key }{ ac_define_class };
+    return $Simo::ac_define_class{ $class }{ $key };
 }
 
 # Get and set accessor opt
@@ -153,15 +188,6 @@ sub _SIMO_ac_opt{
     return $Simo::info{ class }{ $class }{ ac }{ $key }{ opt };
 }
 
-# Check whether default is updated.
-sub _SIMO_obj_updated{
-    my ( $self, $key, $val ) = @_;
-    if( defined( $val ) ){
-        $Simo::info{ obj }{ $self }{ ac }{ $key }{ updated } = $val;
-    }
-    return $Simo::info{ obj }{ $self }{ ac }{ $key }{ updated };
-}
-
 # Helper to get acsessor info;
 sub _SIMO_ac_info {
     package DB;
@@ -186,7 +212,7 @@ Simo - Very simple framework for Object Oriented Perl.
 
 =head1 VERSION
 
-Version 0.01
+Version 0.0205
 
 =cut
 
@@ -505,22 +531,6 @@ If you use hash_force option, you convert list to hash ref
 
 =cut
 
-=head2 scalar context and list context
-
-You can call accessor in scalar context and list context.
-
-Accessor is designed to suit context.
-
-You call accessor in scalar context, you get scalar value( scalar, array ref, hash ref, object, etc )
-
-    my $ary = $book->authors; # you get array ref
-
-You calls accessor in list context, you get list.
-
-    my @ary = $book->authors; # you get list
-
-=cut
-
 =head1 EXPORT
 
 This class exports ac function. you can use ac function to implement accessor. 
